      subroutine richardson(psurf,ust,ttlev,qvlev,ulev,vlev,nuvz,
     +sigmahalf,ptop,p00,ts0,ta,hf,h,wst,hmixplus,ix,jy)

C                             i    i    i     i    i    i    i
C      i   i  i   i   i  o  o     o
*****************************************************************************
* $Id: richardson.f,v 1.1 2006/08/09 13:43:27 paul Exp paul $
* $Log: richardson.f,v $
* Revision 1.1  2006/08/09 13:43:27  paul
* Initial revision
*
*                                                                           *
*     Calculation of mixing height based on the critical Richardson number. *
*     Calculation of convective time scale.                                 *
*     For unstable conditions, one iteration is performed. An excess        *
*     temperature (dependent on hf and wst) is calculated, added to the     *
*     temperature at the lowest model level. Then the procedure is repeated.*
*                                                                           *
*     Author: A. Stohl                                                      *
*                                                                           *
*     22 August 1996                                                        *
*                                                                           *
*     Literature:                                                           *
*     Vogelezang DHP and Holtslag AAM (1996): Evaluation and model impacts  *
*     of alternative boundary-layer height formulations. Boundary-Layer     *
*     Meteor. 81, 245-269.                                                  *
*                                                                           *
*     Update: 1999-02-01 by G. Wotawa                                       *
*                                                                           *
*     Two meter level (temperature, humidity) is taken as reference level   *
*     instead of first model level.                                         *
*     New input variables tt2, td2 introduced.                              *
* ------------------------------------------------------------------        *
*****************************************************************************
*                                                                           *
* Variables:                                                                *
* h                          mixing height [m]                              *
* hf                         sensible heat flux                             *
* psurf                      surface pressure at point (xt,yt) [Pa]         *
* tv                         virtual temperature                            *
* wst                        convective velocity scale                      *
*                                                                           *
* Constants:                                                                *
* ric                        critical Richardson number                     *
*                                                                           *
*****************************************************************************

      include 'includepar'

      integer i,k,itmax,iter,ix,jy,nuvz
      real tv,tvold,zref,z,zold,pint,pold,theta,thetaref,ri
      real sigmahalf(nuvz),ulev(nuvz),vlev(nuvz),hf,wst,tt2,td2,ew
      real psurf,ust,ttlev(nuvz),qvlev(nuvz),h,const,ric,b,excess,bs
      real thetaold,zl,ul,vl,thetal,ril,hmixplus,wspeed,bvfsq,bvf
      real ptop,p00,ts0,ta,thetastar,obukhov,deltat,zmlev1,psih
      real theta1,theta2,zl1,zl2,thetam
      parameter (const=r_air/ga, ric=0.25, b=100., bs=8.5, itmax=3)
      
      real geopot1,sigmatoh,wm,riold
      real ttvlev1,rhoa,r1,fract
      parameter (r1 = 0.74)

      geopot1 = sigmatoh(sigmahalf(1),psurf,ptop,p00,ts0,ta)
      zmlev1 = sigmatoh(sigmahalf(2),psurf,ptop,p00,ts0,ta)-geopot1
	
*
* compute 2 m temperature from temperature at level 1 
* (applying Monin Obukhov similarity)
*                                                 

      ttvlev1 = ttlev(1) * ( 1. + 0.61 * qvlev(1) ) ! qvlev is mixing ratio
      rhoa = psurf / (r_air*ttvlev1)
      theta = ttlev(1)*(100000./psurf)**kappa
      ust = max( 1.e-4, ust )
      thetastar = hf/(rhoa*cpa*ust)
      if (abs(thetastar) .gt. 1.e-10) then
         obukhov = theta*ust**2 / (karman*ga*thetastar)
         deltat = r1*thetastar*( log(zmlev1/2.) - psih(zmlev1,obukhov) +
     &            psih(2.,obukhov) ) / kappa
      else  ! neutral conditions
         deltat = 0.
      endif
      tt2 = ttlev(1) + 0.0098*(zmlev1-2.) - deltat
!!	print*,'rich tt2_',deltat,tt2,thetastar
!!	print*,'rich tt2a',r1,thetastar,zml1,psih(zml1,obukhov)
!!      print*,'rich tt2b',   psih(2.,obukhov),obukhov,theta,ust,karman
      
      excess = 0.
      iter = 0
 
C Compute virtual temperature and virtual potential temperature at
C reference level (2 m)
******************************************************************
 
30    continue
      iter = iter+1
      pold = psurf
      tvold = tt2*( 1. + 0.61 * qvlev(1) )
      zold = 2.0
      zref = zold
      thetaref = tvold*(100000./pold)**kappa + excess
!!	print*,'rich thref', tvold,pold,r_air,cpa,excess
      thetaold = thetaref
      riold = 0.
 
 
C Integrate z up to one level above zt
**************************************
 
      do 10 k = 2,nuvz
        pint  = sigmahalf(k)*(psurf-ptop) + ptop  ! pressure on model layers
        wm = qvlev(k) / ( 1.-qvlev(k) )
        tv = ttlev(k)*( 1. + 0.378 * wm/(wm+0.622) )
        z = sigmatoh(sigmahalf(k),psurf,ptop,p00,ts0,ta) - geopot1

C        if (abs(tv-tvold).gt.0.2) then
C          z = zold+const*log(pold/pint)*(tv-tvold)/log(tv/tvold)
C        else
C          z = zold+const*log(pold/pint)*tv
C        endif
 
        theta = tv*(100000./pint)**kappa
 
 
Calculate Richardson number at each level
*****************************************
 
        ri = ga/thetaref * (theta-thetaref) * (z-zref) /
     &  max( (ulev(k)-ulev(2))**2 + (vlev(k)-vlev(2))**2 + b*ust**2,
     &        0.1)
 
C  addition of second condition: MH should not be placed in an
C  unstable layer (PS / Feb 2000)

        if (ri.gt.ric .and. thetaold.lt.theta) goto 20
        
        riold = ri
        tvold = tv
        pold = pint
        thetaold = theta
        zold = z
10    continue
      if (k .gt. nuvz) k = nuvz

20    continue ! MH found
 
 
C Find exact MH by interpolating between levels found
*****************************************************

      zl1 = zold
      theta1 = thetaold
      do 15 i = 1,20
	  fract = float(i)/20.
        zl = zold + fract * (z-zold)
        if (k.gt.nuvz) stop 'k > nuz (richardson)'
        ul = ulev(k-1) + fract * (ulev(k)-ulev(k-1))
        vl = vlev(k-1) + fract * (vlev(k)-vlev(k-1))
        thetal = thetaold + fract *( theta-thetaold)
        ril = ga/thetaref * (thetal-thetaref) * (zl-zref)/
     &        max( (ul-ulev(2))**2 + (vl-vlev(2))**2 + b*ust**2, 0.1 )
        zl2 = zl
        theta2 = thetal
        if (ril .gt. ric) goto 25
        zl1 = zl
        theta1 = thetal
15    continue
 
25    continue
      h = zl
      thetam = 0.5 * (theta1 + theta2)
      wspeed = sqrt(ul**2 + vl**2)                    ! Wind speed at z = hmix
      bvfsq = (ga/thetam) * (theta2-theta1) / (zl2-zl1) ! Brunt-Vaisala freq^2
                                                  ! at z = hmix
      if (zl2-zl1 .eq. 0.) stop 'zl2-zl0  =  0!!'

C Under stable conditions, limit the maximum effect of the subgrid-scale 
C topography by the maximum lifting possible from the available kinetic energy
********************************************************************************

      if (bvfsq .le. 0.) then
        hmixplus = 9999.
      else
        bvf = sqrt(bvfsq)
        hmixplus = wspeed / bvf * convke         ! keconv  =  kinetic energy
      endif                                      ! used for lifting

 
C Calculate convective velocity scale
*************************************
 
      if (hf .lt. 0.) then
        wst = (-h * ga/thetaref * hf/cpa)**0.333
        excess = -bs * hf / (cpa*wst)
        if (iter .lt. itmax) goto 30
      else
        wst = 0.
      endif

      end
